<?php
/**
 * The Giapeto script to edit a page, or if no page_id passed, to create a new
 * page.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: giapeto/edit.php,v 1.52 2005/07/24 21:21:37 chuck Exp $
 */

@define('GIAPETO_BASE', dirname(__FILE__));
require_once GIAPETO_BASE . '/lib/base.php';
require_once 'Horde/Variables.php';

/* Form libraries */
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';

$giapeto = &Giapeto::singleton();

/* Set up the form variables. */
$vars = &Variables::getDefaultVariables();
$page_id = $vars->get('page_id');
$is_root_page = (!empty($page_id) ? $giapeto->isRootPage($page_id) : false);
$editing = !empty($page_id);

/* Check if user authorised for this. */
$check_page_id = !empty($page_id) ? $page_id : $vars->get('parent_id');
if (!$giapeto_perms->checkPermissions($check_page_id, 'page', PERMS_EDIT)) {
    $notification->push(_("You are not authorized for this action."), 'horde.error');
    $url = $giapeto->getIdUrl($check_page_id, 1);
    header('Location: ' . $url);
    exit;
}

/* Set up the form object. */
$form = &Horde_Form::singleton('', $vars, ($editing ? _("Edit Page") : _("Create a New Page")));

/* Check if the user is allowed to add more pages. */
if (!empty($page_id) && !empty($conf['site']['max_pages']) &&
    $giapeto->getPageCount() >= $conf['site']['max_pages']) {
    $notification->push(_("Maximum number of pages already present."), 'horde.error');
    $page_path = $giapeto->getPagePath($vars->get('parent_id'));
    $url = $giapeto->getCategoryUrl($page_path, 1);
    header('Location: ' . $url);
    exit;
}

/* Check if form is being edited and not yet submitted. */
if ($page_id && !$form->isSubmitted()) {
    $page = $giapeto->getPage($page_id);
    $vars = array();
    $page->getPageData($vars);
    $vars['page_id'] = $page_id;
    $vars['name'] = $page->getShortName();
    $vars['parent_path'] = $giapeto->getParentName($page);
    if (is_null($vars['parent_path'])) {
        $vars['parent_path'] = '-';
    } else {
        $vars['parent_path'] = _("Home") . $giapeto->getCategoryUrl($vars['parent_path'], -1);
    }

    $vars = &new Variables($vars);
    $form->setVars($vars);
}

$no_errors = true;

/* Set up the form. */
$form->setButtons(_("Save"), true);
$form->addHidden('', 'page_id', 'int', $editing);
/* If root page then read-only and not required. */
if ($is_root_page) {
    $form->addVariable(_("Page name"), 'name', 'text', false, true);
} else {
    $form->addVariable(_("Page name"), 'name', 'text', true, false, _("Characters ':' and '/' are not allowed."), array('|^[\w][^:/]+$|'));
}
$form->addVariable(_("Template"), 'template', 'enum', false, false, null, array($giapeto->listTemplates(), _("-- Inherit Parent Template --")));
if ($editing) {
    /* Editing so just show the path and add subpage ordering. */
    $form->addVariable(_("Subpage of"), 'parent_path', 'text', false, true);
    $subpages = $giapeto->getChildren($page_id);
    if (count($subpages) > 1) {
        $form->addVariable(_("Order of subpages"), 'subpage_order', 'sorter', false, false, null, array($subpages, 8, _("Available subpages:")));
    }
} else {
    /* Not editing, path can be set. */
    $sitetree = $giapeto->getSiteTree('cat_ids', -1, true, true);
    $form->addVariable(_("Subpage of"), 'parent_id', 'enum', false, false, null, array($sitetree));
}

if ($form->validate($vars)) {
    $form->getInfo($vars, $info);
    $page_id = $info['page_id'];

    /* Check if editing this page. */
    if ($editing) {
        $page = $giapeto->getPage($page_id);
        $page_path = $page->name;

        /* Check if any subpages and if a different order has been set. */
        $subpages = array_keys($subpages);
        if (count($subpages) > 1 && $subpages != $info['subpage_order']) {
            /* Different subpage order set. */
            $reorder = $giapeto->reorderSubpages($page_id, $info['subpage_order']);
        }

        /* Check if the page has been renamed. */
        $rename = false;
        if (($page->getShortName() != $info['name']) &&
            !$is_root_page) {
            $rename = $giapeto->_pages->rename($page, $info['name']);
            if (is_a($rename, 'PEAR_Error')) {
                Horde::logMessage($rename, __FILE__, __LINE__, PEAR_LOG_ERR);
                $notification->push(sprintf(_("Can not rename the page. %s"), $rename->getMessage()), 'horde.error');
                $no_errors = false;
            } else {
                /* Page renamed ok, now change block VFS paths. */
                $info['blocks'] = $page->getRawBlocks();
                $old_vfs_path = $giapeto->getCategoryUrl($page_path, -1);
                $page_path = $giapeto->getParentName($page) . ':' . $info['name'];
                $new_vfs_path = $giapeto->getCategoryUrl($page_path, -1);

                /* Change each block's VFS path. */
                foreach ($info['blocks'] as $block_id => $block_val) {
                    if (!empty($block_val['vfs_id'])) {
                        $GLOBALS['vfs']->rename(GIAPETO_VFS_PATH . $old_vfs_path, $block_val['vfs_id'], GIAPETO_VFS_PATH . $new_vfs_path, $block_val['vfs_id']);
                        $info['blocks'][$block_id]['vfs_path'] = $new_vfs_path;
                    }
                }
            }
        }
    } else {
        /* Set up the new page. */
        $parent_page_path = $giapeto->getPagePath($info['parent_id']);
        $page_path = $parent_page_path . ':' . $info['name'];
        $page = &$giapeto->newPage($page_path);

        /* Start a new page with a default page layout. */
        $info['layout'] = '[x/x/x]';
    }

    if ($no_errors) {
        /* Set the layout and the template info to the page object. */
        $page->setPageTemplate($info['template']);
        if (isset($info['layout'])) {
            $page->setPageLayout($info['layout']);
        }
        $result = $page->save();
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($results, __FILE__, __LINE__, PEAR_LOG_ERR);
            if ($page_id) {
                $notification->push(sprintf(_("Page modification failed. %s"), $results->getMessage()), 'horde.error');
            } else {
                $notification->push(sprintf(_("Creating new page failed. %s"), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("Page saved."), 'horde.success');
            $url = $giapeto->getCategoryUrl($page_path, 1);
            header('Location: ' . $url);
            exit;
        }
    }
}

/* Render the form. */
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'edit.php', 'post');

$template = &new Horde_Template();
$template->set('menu', Giapeto::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify'), array('listeners' => 'status')));
$template->set('main', $main);

require GIAPETO_TEMPLATES . '/common-header.inc';
echo $template->fetch(GIAPETO_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
