<?php

/* Giapeto VFS path. */
@define('GIAPETO_VFS_PATH', '.horde/giapeto/' . $conf['site']['name']);

/* These are the constants used for actions. */

/**
 * Show a page.
 */
define('G_SHOWPAGE', 100);

/**
 * The Giapeto:: class provides the Horde Giapeto system.
 *
 * Copyright 2002-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: giapeto/lib/Giapeto.php,v 1.213 2005/09/13 21:46:46 selsky Exp $
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @package Giapeto
 */
class Giapeto {

    /**
     * Pointer to a DataTree instance to manage the pages.
     *
     * @var DataTree
     */
    var $_pages;

    /**
     * The root page id.
     */
    var $_root_page_id;

    /**
     * The current page id.
     */
    var $_page_id;

    /**
     * The current action.
     */
    var $_action_id;

    /**
     * Constructor.
     */
    function Giapeto()
    {
        global $conf;

        if (empty($conf['datatree']['driver']) ||
            $conf['datatree']['driver'] == 'none') {
            Horde::fatal(sprintf(_("You must configure a DataTree backend in horde/config/conf.php to use %s."), $GLOBALS['registry']->get('name')), __FILE__, __LINE__);
        }

        $driver = $conf['datatree']['driver'];
        $this->_pages = &DataTree::singleton($driver,
                                             array_merge(Horde::getDriverConfig('datatree', $driver),
                                                         array('group' => 'giapeto.pages.' . $conf['site']['name'])));

        /* Auto-create the root page if it doesn't exist, but die if
         * we encounter too many problems trying to do so. */
        $this->_root_page_id = $this->_pages->getId('__root');
        if (is_a($this->_root_page_id, 'PEAR_Error')) {
            $rootpage = &$this->newPage('__root');
            $result = $rootpage->save();
            if (is_a($result, 'PEAR_Error')) {
                Horde::fatal($result, __FILE__, __LINE__);
            }

            $this->_root_page_id = $this->_pages->getId('__root');
            if (is_a($this->_root_page_id, 'PEAR_Error')) {
                Horde::fatal($this->_root_page_id, __FILE__, __LINE__);
            }
        }

        $this->_page_id = $this->getThisPageId();
    }

    /**
     * Attempts to return a reference to a concrete Giapeto instance. It will
     * only create a new instance if no Giapeto instance currently exists.
     *
     * This method must be invoked as: $var = &Giapeto::singleton()
     *
     * @return Giapeto  The concrete Giapeto reference, or false on an error.
     */
    function &singleton()
    {
        static $page;

        if (!isset($page)) {
            require_once GIAPETO_BASE . '/lib/Page.php';
            $page = new Giapeto();
        }

        return $page;
    }

    /**
     * Return a new page object.
     *
     * @param string $page_name  The page's name.
     *
     * @return DataTreeObject_Page  A new page object.
     */
    function &newPage($page_name)
    {
        if (empty($page_name)) {
            return PEAR::raiseError('Page names must be non-empty');
        }
        $page = &new DataTreeObject_Page($page_name);
        $page->setPageOb($this);
        return $page;
    }

    /**
     * Return a DataTreeObject_Page object corresponding to the page id
     * requested.
     *
     * @param string $page_id  The page_id of the page to retrieve.
     *
     * @return DataTreeObject_Page  Page object for page id requested.
     */
    function &getPage($page_id)
    {
        /* Cache of previous retrieved pages. */
        static $pagesCache;

        if (!is_array($pagesCache)) {
            $pagesCache = array();
        }

        if (isset($pagesCache[$page_id])) {
            return $pagesCache[$page_id];
        }

        $page = &$this->_pages->getObjectById($page_id, 'DataTreeObject_Page');
        if (is_a($page, 'PEAR_Error')) {
            return $page;
        }

        $pagesCache[$page_id] = &$page;
        $pagesCache[$page_id]->setPageOb($this);

        return $pagesCache[$page_id];
    }

    /**
     * Parse the format string to create the html table structure:
     *   x - replace with %s for use in vprintf
     *   / - close the tr and start a new one
     *   [ - start a new table
     *   ] - close the table
     *   | - close the td and open a new one
     *
     * @param string $pf     The format string, eg. x/[x|x].
     * @param string $width  Set the table width, default to 100%.
     *
     * @return string  Layout expanded to html and %s for vsprintf.
     */
    function parsePageLayout($pf, $width = '100%%')
    {
        if (isset($pf) && !empty($pf)) {
            $search = array('x', '/', '[', ']', '|');
            // Intentional double % (%%) to be used in vsprintf later
            // without problems.
            $replace = array('%s',
                             '</td></tr><tr valign="top"><td>',
                             '<table height="100%%" width="' . $width . '" border="0" cellpadding="2" cellspacing="0"><tr valign="top"><td>',
                             '</td></tr></table>',
                             '</td><td>');
            $pf = str_replace($search, $replace, $pf);
            return $pf;
        }
        return false;
    }

    /**
     * Clean up the page format string by replacing some redundant logic with
     * equivalent streamlined representation. Keep on iterating until nothing
     * can be changed.
     *
     * @param string $ugly_format  The format string to streamline.
     *
     * @return string  Streamlined format string.
     */
    function optimizePageLayout($ugly_format)
    {
        $bad = array('[x]', '//', '/]', '[/', '||', '|]', '[|', '[]',
                     'x/[x/x]', '[x/x]/x', '[x|x]|x', 'x|[x|x]');
        $good = array('x', '/', ']', '[', '|', ']', '[', '', 'x/x/x',
                      'x/x/x', 'x|x|x', 'x|x|x');

        /* and get rid of the ugly... */
        $format = str_replace($bad, $good, $ugly_format);

        if ($format == $ugly_format) {
            return $format;
        } else {
            return Giapeto::optimizePageLayout($format);
        }
    }

    /**
     * Store page data to the pages catalog. Handles both adding of new pages
     * and updating of existing pages. When adding a new page the page must
     * first be created with &Giapeto::newPage(), and have any initial data
     * added to it, before this function is called.
     *
     * @param DataTreeObject_Page $page  The new or existing page object.
     *
     * @return mixed  True for success or a PEAR error on failure.
     */
    function storePage(&$page)
    {
        if (!is_a($page, 'DataTreeObject_Page')) {
            return PEAR::raiseError('Pages must be DataTreeObject_Page objects or extend that class.');
        }

        if ($this->_pages->exists($page)) {
            return $this->_pages->updateData($page);
        } else {
            return $this->_pages->add($page);
        }
    }

    /**
     * Remove a page from the pages system permanently.
     *
     * @param DataTreeObject_Page $page  The page to remove.
     *
     * @return mixed  True for success or a PEAR error on failure.
     */
    function removePage($page)
    {
        if (!is_a($page, 'DataTreeObject_Page')) {
            return PEAR::raiseError('Pages must be DataTreeObject_Page objects or extend that class.');
        }

        $parent_path = $this->getParentName($page);
        $vfs_path = GIAPETO_VFS_PATH . $this->getCategoryUrl($parent_path, -1);
        $page_end_name = $page->getShortName();

        /* TODO: TO BE FIXED! Real hack, so that all pages in a subtree can
         * be deleted in one go. First create the folder we are deleting. */
        $create = $GLOBALS['vfs']->createFolder($vfs_path, $page_end_name);

        /* Delete page parts in VFS and all within the subtree. */
        $delete = $GLOBALS['vfs']->deleteFolder($vfs_path, $page_end_name, true);

        /* Delete any permissions for this page. */
        global $giapeto_perms;
        if ($giapeto_perms->exists('giapeto:pages:' . $page->getId())) {
            $perm = $giapeto_perms->getPermission('giapeto:pages:' . $page->getId());
            $delete = $giapeto_perms->removePermission($perm);
        }

        /* Delete from Horde_DataTree and return. */
        return $this->_pages->remove($page, true);
    }

    /**
     * Move a page from one parent to another.
     *
     * @param integer $page_id  The page id to move.
     * @param integer $newcat   The id of the new parent page.
     *
     * @return string  The new page name.
     */
    function movePage($page_id, $new_parent_id)
    {
        /* Get the page object and its blocks being moved */
        $page = $this->getPage($page_id);
        $page_blocks = $page->getRawBlocks();

        /* Get the name of this object */
        $page_name = $page->name;

        /* Get the name of the new parent */
        $new_parent_name = $this->getPagePath($new_parent_id);

        /* Fix up the blocks so that they get the new path */
        $old_vfs_path = $this->getCategoryUrl($page_name, -1);
        $new_vfs_path = $this->getIdUrl($new_parent_id, -1) . ($this->isRootPage($new_parent_id) ? '' : '/') . $this->getPagePathEnd($page_name);
        foreach ($page_blocks as $block_id => $block_val) {
            $GLOBALS['vfs']->rename(GIAPETO_VFS_PATH . $old_vfs_path, $block_val['vfs_id'], GIAPETO_VFS_PATH . $new_vfs_path, $block_val['vfs_id']);
            $page_blocks[$block_id]['vfs_path'] = $new_vfs_path;
        }

        /* Store the block data into the page object and save the page */
        $page->data['blocks'] = $page_blocks;
        $page->save();

        /* Do the move */
        $this->_pages->move($page_name, $new_parent_name);

        /* Return the new page name */
        return $new_parent_name;
    }

    /**
     * Gets the page id of the root page.
     *
     * @return integer  The page id of the root page.
     */
    function getRootPageId()
    {
        return $this->_root_page_id;
    }

    /**
     * Checks if a page id is the root page for the site.
     *
     * @param integer $page_id  Page id to check, default to current page.
     *
     * @return boolean  Whether the page id is root or not.
     */
    function isRootPage($page_id = null)
    {
        /* Fetch the current page id if one is not passed. */
        if (is_null($page_id)) {
            $page_id = $this->_page_id;
        }
        return ($page_id == $this->_root_page_id);
    }

    function setActionId($action_id)
    {
        $this->_action_id = $action_id;
    }

    function getActionId()
    {
        return $this->_action_id;
    }

    function getPageId($page)
    {
        if (is_a($page, 'DataTreeObject_Page')) {
            $page = $page->getName();
        }
        return $this->_pages->getId($page);
    }

    function listTemplates()
    {
        global $vfs;

        if (!$vfs->exists(GIAPETO_VFS_PATH, 'templates')) {
            $vfs->createFolder(GIAPETO_VFS_PATH, 'templates');
            $vfs->write(GIAPETO_VFS_PATH . '/templates', 'default_template', GIAPETO_BASE . '/scripts/data/default_template');
        }

        $templates = $vfs->listFolder(GIAPETO_VFS_PATH . '/templates');
        if (is_a($templates, 'PEAR_Error')) {
            return $templates;
        }

        $template_names = array_keys($templates);
        asort($template_names);

        $templates = array();
        foreach ($template_names as $template_name) {
            $templates[$template_name] = $template_name;
        }

        return $templates;
    }

    function getPageTemplate($page)
    {
        global $vfs;

        /* Check for this page's template. */
        if (!empty($page->data['page']['template'])) {
            $template = $page->data['page']['template'];
        } else {
            /* Then check for any parents' templates. */
            $parents = explode(':', $this->getParentIdString($page));
            while (($parent_id = array_pop($parents)) && empty($template)) {
                $parent = &$this->getPage($parent_id);
                if (!empty($parent->data['page']['template'])) {
                    $template = $parent->data['page']['template'];
                }
            }
        }

        /* Make sure the template requested actually exists. */
        if (empty($template) || !$vfs->exists(GIAPETO_VFS_PATH . '/templates', $template)) {
            $templates = $this->listTemplates();
            if (is_a($templates, 'PEAR_Error')) {
                Horde::fatal($templates, __FILE__, __LINE__, true);
            }
            $templates = array_values($templates);
            $template = $templates[0];
        }
        return $template;
    }

    /**
     * Get the full path of the page.
     *
     * @param integer $page_id  The page_id.
     *
     * @return string  The page path.
     */
    function getPagePath($page_id)
    {
        return $this->_pages->getName($page_id);
    }

    /**
     * Given a path return the end name.
     *
     * @param string $page_path  The page path.
     *
     * @return string  The end name of the page.
     */
    function getPagePathEnd($page_path)
    {
        if ($page_path == '__root') {
            return _("Home");
        }
        return array_pop(explode(':', $page_path));
    }

    /**
     * Get the page id of the parent.
     *
     * @param mixed $page  The name of the page or the actual DataTreeObject for
     *                     which to retrieve the parent id.
     *
     * @return int  The page id of the parent.
     */
    function getParentId($page)
    {
        if (is_a($page, 'DataTreeObject_Page')) {
            $page = $page->getName();
        }
        return $this->_pages->getParent($page);
    }

    /**
     * Get the parent page name.
     *
     * @param mixed $page  The name of the page or the actual DataTreeObject for
     *                     which to retrieve the parent name.
     *
     * @return string  The name of the parent.
     */
    function getParentName($page)
    {
        $parent = $this->getParentId($page);
        return $this->_pages->getName($parent);
    }

    function getParents($child = null)
    {
        return $this->_pages->getParents($child);
    }

    /**
     * Get the id string of parents.
     *
     * @param mixed $page  Either the page name of the child or the page object.
     *
     * @return string  The ids of the parents separated by ':'.
     */
    function getParentIdString($page = null)
    {
        if (is_null($page)) {
            $page = $this->_pages;
        }
        if (is_a($page, 'DataTreeObject_Page')) {
            $page = $page->getName();
        }

        return $this->_pages->getParentIdString($page);
    }

    function getFormattedNavPath($page_id)
    {
        $formattednavpath = array();
        $nav_path = $this->getNavPath($page_id);
        foreach ($nav_path as $page) {
            $formattednavpath[] = Horde::link($page['url']) . $page['text'] . '</a>';
        }
        return $formattednavpath;
    }

    function getNavPath($page_id)
    {
        static $nav_path = array();
        if (isset($nav_path[$page_id])) {
            return $nav_path[$page_id];
        }

        $page_path = $this->getPagePath($page_id);
        $nav_pages = explode(':', $page_path);
        array_pop($nav_pages);

        $path = '';
        $nav_path[$page_id] = array();
        foreach ($nav_pages as $page_name) {
            $path .= (empty($path)? '' : ':') . $page_name;
            $url = $this->getCategoryUrl($path);
            if ($page_name == '__root') {
                $page_name = _("Home");
            }
            $nav_path[$page_id][] = array('url' => $url, 'text' => $page_name);
        }
        return $nav_path[$page_id];
    }

    function getFormattedSubPages($page_id = DATATREE_ROOT)
    {
        $formattedsubpages = array();
        $subpages = $this->getSubPages($page_id);
        foreach ($subpages as $page) {
            $formattedsubpages[] = Horde::link($page['url']) . $page['text'] . '</a>';
        }
        return $formattedsubpages;
    }

    function getSubPages($page_id = DATATREE_ROOT)
    {
        static $subpages = array();
        if (isset($subpages[$page_id])) {
            return $subpages[$page_id];
        }

        $sp = $this->getChildren($page_id);
        $sp = $GLOBALS['giapeto_perms']->checkPermissions($sp, 'subpages', PERMS_SHOW);

        $subpages[$page_id] = array();
        foreach ($sp as $sp_key => $sp_val) {
            $cat = $this->_pages->getName($sp_key);
            $url = $this->getCategoryUrl($cat);
            $subpages[$page_id][] = array('id'   => $sp_key,
                                          'url'  => $url,
                                          'text' => $sp_val);
        }
        return $subpages[$page_id];
    }

    /**
     * Returns the children to a page.
     *
     * @param integer $parent  The page id of the parent leaf.
     *
     * @return array  An array of child pages.
     */
    function getChildren($parent = DATATREE_ROOT)
    {
        /* Get one level of children. */
        $parent_name = $this->_pages->getName($parent);
        $children = $this->_pages->get(DATATREE_FORMAT_FLAT, $parent_name, true, DATATREE_ROOT, 1);

        /* Do not include the parent level in the array. */
        unset($children[$parent]);

        /* Loop through and get short name for each full page name. */
        foreach ($children as $key => $name) {
            $children[$key] = DataTree::getShortName($name);
        }

        return $children;
    }

    /**
     * Returns a site tree, either of the whole site or a part of it.
     *
     * @param constant $format    Format of the tree.
     * @param integer $startleaf  The id of the leaf from which we start the
     *                            export tree.
     * @param string $editing     Whether the site tree is used in an editing
     *                            situation or not, so that the correct
     *                            permissions are checked.
     *
     * @return mixed  The site tree or a PEAR_Error on failure.
     */
    function getSiteTree($format = null, $startleaf = DATATREE_ROOT, $editing = false, $no_root = false, $filter_out = '')
    {
        global $giapeto_perms;

        $tree = array();

        $startleaf_name = $this->_pages->getName($startleaf);
        $base_indent = substr_count($startleaf_name, ':');

        $pages = $this->_pages->get(DATATREE_FORMAT_FLAT, $startleaf_name, true);

        /* Don't show the root. */
        if ($no_root) {
            unset($pages[$startleaf]);
        }

        if ($editing) {
            $pages = $giapeto_perms->checkPermissions($pages, 'sitetree', PERMS_EDIT);
        } else {
            $pages = $giapeto_perms->checkPermissions($pages, 'sitetree', PERMS_SHOW);
        }

        /* Get length of the $filter_out var in case needs to be checked. */
        $len = strlen($filter_out);

        /* Output in the requested format. */
        switch ($format) {
        case 'cat_ids':
            foreach ($pages as $k => $v) {
                if ($filter_out && $filter_out == substr($v, 0, $len)) {
                    continue;
                }
                $tree[$k] = str_repeat('.. ', substr_count($v, ':') - $base_indent);
                $end = $this->getPagePathEnd($v);
                $tree[$k] .= $end;
            }
            break;

        case 'tag':
            $i = 0;
            $thispage = $this->_page_id;
            foreach ($pages as $k => $v) {
                if ($filter_out && $filter_out == substr($v, 0, $len)) {
                    continue;
                }
                $tree[$i]['url'] = $this->getCategoryUrl($v);
                $tree[$i]['indent'] = str_repeat('.. ', substr_count($v, ':') - $base_indent);
                $tree[$i]['pagename'] = $this->getPagePathEnd($v);
                $tree[$i]['selected'] = $thispage == $this->getPageId($v) ? ' selected="selected"' : '';
                $i++;
            }
            break;

        case 'cat_id_strings':
            foreach ($pages as $k => $v) {
                $tree[$this->getParentIdString($v) . ':' . $k] = $v;
            }
            break;

        default:
            return $pages;
        }

        return $tree;
    }

    /**
     * Not efficient because of the perms check. Don't use outside of
     * checking for max_pages.
     */
    function getPageCount($startleaf = DATATREE_ROOT, $perms_check = false)
    {
        /* Fetch the tree. */
        $startleaf_name = $this->_pages->getName($startleaf);
        $tree = $this->_pages->get(DATATREE_FORMAT_FLAT, $startleaf_name, true);

        /* Do a permissions check if required to count only allowed
         * pages. */
        if ($perms_check) {
            $tree = $GLOBALS['giapeto_perms']->checkPermissions($tree, 'sitetree', PERMS_EDIT);
        }

        return count($tree);
    }

    function getUrlCategory($url_path)
    {
        $url_parts = explode('/', $url_path);
        $cat_parts = array();
        foreach ($url_parts as $up) {
            if (!empty($up)) $cat_parts[] = $up;
        }
        if (empty($cat_parts) || $cat_parts[0] != '__root') {
            array_unshift($cat_parts, '__root');
        }

        return join(':', $cat_parts);
    }

    function getUrlCategoryId($url_path)
    {
        $cat_name = $this->getUrlCategory($url_path);
        return $this->_pages->getId($cat_name);
    }

    /**
     * Returns the relative url for the category path, formatted
     * according to what is configured method of passing the page id.
     *
     * @param string $cat_path  The Horde_DataTree category path.
     * @param integer $full     What type of url to return:
     *                            -1 = just the url representation of the
     *                                 category.
     *                             0 = default, url for use in links.
     *                             1 = full url for use in redirects.
     *
     * @return string  The category path as a url.
     */
    function getCategoryUrl($cat_path, $type = 0)
    {
        /* Cache these category => url translations. */
        static $url = array();
        $id = $cat_path . '_' . $type;

        /* Return if already exists a cat_path/type combination. */
        if (isset($url[$id])) {
            return $url[$id];
        }

        $path_parts = explode(':', $cat_path);
        if ($path_parts[0] == '__root') {
            unset($path_parts[0]);
        }

        /* Check if only need category path as a url. */
        if ($type == -1) {
            $url[$id] = '/' . implode('/', $path_parts);
            return $url[$id];
        }

        /* Doing a full url so urlencode() the parts. */
        array_walk($path_parts, array('Giapeto', '_urlencodePathParts'));

        /* Join the parts with a prefixed '/'. */
        $cat_url = implode('/', $path_parts);

        /* Format the url according to the url method selected. */
        switch ($GLOBALS['conf']['site']['url_type']) {
        case 'get':
            return Horde::applicationUrl('pages.php?page_id=' . $cat_url, $type);

        case 'path_info':
            return Horde::applicationUrl('pages.php/' . $cat_url, $type);

        case 'rewrite':
            return Horde::applicationUrl($cat_url, $type);
        }
    }

    /**
     * Returns the url from a Horde_DataTree id. Actually fetches the
     * page and calls getCategoryUrl().
     *
     * @param integer $cat_id  The Horde_DataTree category path.
     * @param integer $full    What type of url to return:
     *                           -1 = just the url representation of the
     *                                category.
     *                            0 = default, url for use in links.
     *                            1 = full url for use in redirects.
     *
     * @return string  The url of the category id.
     */
    function getIdUrl($cat_id, $type = 0)
    {
        $cat_path = $this->_pages->getName($cat_id);
        return $this->getCategoryUrl($cat_path, $type);
    }

    /**
     * Returns the current page id based on the configured method of passing
     * giapeto page urls.
     *
     * @return int  The page id of the current page.
     * @todo  Insert raising of errors when requesting inexistant pages.
     */
    function getThisPageId()
    {
        if (!isset($this->_page_id)) {
            /* Figure out the configured url passing method. */
            switch ($GLOBALS['conf']['site']['url_type']) {
            case 'get':
            case 'rewrite':
                $page_url = Util::getFormData('page_id');
                break;

            case 'path_info':
                $page_url = isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : null;
                break;
            }

            /* Get the page id for this url. */
            $this->_page_id = $this->getUrlCategoryId($page_url);
        }

        return $this->_page_id;
    }

    function reorderSubpages($page_cat_id, $subpages)
    {
        $page_cat_path = $this->_pages->getName($page_cat_id);
        $page_cat_path = $this->getParentIdString($page_cat_path) . ':' . $page_cat_id;
        return $this->_pages->reorder($page_cat_path, $subpages);
    }

    function getBlockTypes()
    {
        static $types = array();
        if (!empty($types)) {
            return $types;
        }

        $types['text'] = _("Text");
        if (in_array('richtext', $GLOBALS['conf']['site']['block_types'])) {
            $types['richtext'] = _("Rich Text");
        }
        if (in_array('image', $GLOBALS['conf']['site']['block_types'])) {
            $types['image'] = _("Image");
        }
        if (in_array('file', $GLOBALS['conf']['site']['block_types'])) {
            $types['file'] = _("File");
        }
        if (in_array('app', $GLOBALS['conf']['site']['block_types'])) {
            $types['app'] = _("Application Block");
        }
        $types['delete'] = _("Empty Block");

        return $types;
    }

    function getUpdatedPages()
    {
        $criteria = array(
            'AND' => array(
                array('field' => 'name', 'op' => '=', 'test' => 'p'),
                array('field' => 'key', 'op' => '=', 'test' => 'announce')));
        return $this->_pages->getByAttributes($criteria);
    }

    function renderPermsTree($start_leaf = DATATREE_ROOT, $current_page_id = null)
    {
        require_once 'Horde/Tree.php';

        /* Admin always starts from root. */
        if (Auth::isAdmin()) {
            $start_leaf = DATATREE_ROOT;
        }

        /* If starting leaf is DATATREE_ROOT, do not print the
         * starting leaf. */
        $no_root = ($start_leaf == DATATREE_ROOT);
        $pages = $this->getSiteTree(null, $start_leaf, false, $no_root);

        /* Urls and images. */
        $view = Horde::applicationUrl('perms/index.php');
        $add = Horde::applicationUrl('perms/add.php');
        $add_img = Horde::img('perms.png', _("Add Permissions"), 'hspace="2"', $GLOBALS['registry']->getImageDir('horde'));
        $edit = Horde::applicationUrl('perms/edit.php');
        $edit_img = Horde::img('edit.png', _("Edit Permissions"), 'hspace="2"', $GLOBALS['registry']->getImageDir('horde'));
        $delete = Horde::applicationUrl('perms/delete.php');
        $delete_img = Horde::img('delete.png', _("Delete Permissions"), 'hspace="2"', $GLOBALS['registry']->getImageDir('horde'));

        /* Set up the tree object. */
        $tree = &Horde_Tree::singleton('giapeto_admin', 'javascript');
        $tree->setOption(array('alternate' => true));

        /* Loop through the pages and add the nodes to the tree. */
        foreach ($pages as $page_id => $page_name) {
            $parent_catname = $this->getParentName($page_name);
            if (!is_null($parent_catname) && $parent_catname != DATATREE_ROOT) {
                $parent_id = $this->getPageId($parent_catname);
            } else {
                $parent_id = null;
            }

            /* Make sure the current page is always expanded. */
            $expanded = ($page_id == $current_page_id);

            /* The node label is a clickable link to the permissions view
             * for that node. */
            $node_label = Util::addParameter($view, 'page_id', $page_id);
            $node_label = Horde::link($node_label) . $this->getPagePathEnd($page_name) . '</a>';

            /* Figure out what action links should be with this node. */
            $links = array();
            $perm_name = 'giapeto:pages:' . $page_id;
            $permission = &$GLOBALS['perms']->getPermission($perm_name);
            if (is_a($permission, 'PEAR_Error')) {
                /* No permissions set for this page yet. */
                $links[] = Horde::link(Util::addParameter($add, 'page_id', $page_id), _("Add Permissions")) . $add_img . '</a>';
            } else {
                /* There are permissions for this page, show edit and delete
                 * buttons. */
                $links[] = Horde::link(Util::addParameter($edit, 'page_id', $page_id), _("Edit Permissions")) . $edit_img . '</a>';
                $links[] = Horde::link(Util::addParameter($delete, 'page_id', $page_id), _("Delete Permissions")) . $delete_img . '</a>';
            }

            $tree->addNode($page_id, $parent_id, $node_label, substr_count($page_name, ':'), $expanded, null, $links);
        }

        return Util::bufferOutput(array($tree, 'renderTree'));
    }

    /**
     * Function used in array_walk calls to urlencode elements in an array
     * which are parts of an url.
     *
     * @param array $part
     */
    function _urlencodePathParts(&$part, $key)
    {
        $part = rawurlencode($part);
    }

    /**
     * Build Giapeto's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        global $registry, $giapeto, $giapeto_perms;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();
        $current_page_id = $giapeto->getThisPageId();

        /* Giapeto Pages. */
        if ($giapeto->getActionId() != G_SHOWPAGE) {
            $menu->addArray(array('url' => $giapeto->getIdUrl($current_page_id), 'text' => _("Pages"), 'icon' => 'giapeto.png'));
        }

        /* Giapeto Add Page. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_CREATE)) {
            $url = Horde::applicationUrl('edit.php');
            $url = Util::addParameter($url, 'parent_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("New Page"), 'icon' => 'page_new.png'));
        }

        /* Giapeto Edit Page. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_EDIT)) {
            $url = Horde::applicationUrl('edit.php');
            $url = Util::addParameter($url, 'page_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("Edit Page"), 'icon' => 'page_edit.png'));
        }

        /* Page permissions. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_EDIT)) {
            $url = Horde::applicationUrl('perms/index.php');
            $url = Util::addParameter($url, 'page_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("Page Permissions"), 'icon' => 'page_perms.png'));
        }

        /* Giapeto Page Layout. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_EDIT)) {
            $url = Horde::applicationUrl('layout.php');
            $url = Util::addParameter($url, 'page_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("Page Layout"), 'icon' => 'page_layout.png'));
        }

        /* Giapeto Move Page. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_MOVE) && !$giapeto->isRootPage()) {
            $url = Horde::applicationUrl('move.php');
            $url = Util::addParameter($url, 'page_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("Move Page"), 'icon' => 'page_move.png'));
        }

        /* Giapeto Delete Page. */
        if ($giapeto->getActionId() == G_SHOWPAGE && $giapeto_perms->checkPermissions($current_page_id, 'page', PERMS_DELETE)) {
            $url = Horde::applicationUrl('delete.php');
            $url = Util::addParameter($url, 'page_id', $current_page_id);
            $menu->addArray(array('url' => $url, 'text' => _("Delete Page"), 'icon' => 'page_delete.png'));
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
