<?php

require_once 'Horde/Perms.php';

/** The object can be created. */
define('PERMS_CREATE', 128);

/** The object can be moved. */
define('PERMS_MOVE', 1024);

/**
 * Giapeto Permissions Class
 *
 * $Horde: giapeto/lib/Perms.php,v 1.25 2005/06/07 23:28:49 selsky Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @package Giapeto
 */
class Giapeto_Perms {

    /**
     * Instance of the Horde Perms driver that we pass calls through to as
     * necessary.
     *
     * @var Perms
     */
    var $_perms;

    /**
     * Construct a new Giapeto_Perms object and a Horde Perms object
     * that we wrap around.
     */
    function Giapeto_Perms()
    {
        $this->_perms = &Perms::singleton();
    }

    /**
     * Get the id string of parents.
     *
     * @param  mixed  $in          A page_id or an array of page_id's
     *                             to check permission on.
     * @param  string $filter      What type of check to do.
     * @param  string $permission  What type of permission to check for.
     *
     * @return mixed               Depending on the type of check,
     *                             either return a boolean to indicate
     *                             permission for that page, or a
     *                             filtered out array of page_id's.
     */
    function checkPermissions($in, $filter, $permission = PERMS_READ)
    {
        global $giapeto, $conf;
        static $permsCache;

        $admin = Auth::isAdmin();
        /* Horde admin is always authorised. */
        if ($admin) {
            return $in;
        }

        if (!$conf['site']['perms_extended'] && ($permission == PERMS_CREATE || $permission == PERMS_MOVE)) {
            $permission = PERMS_EDIT;
        }

        $out = array();
        $userID = Auth::getAuth();
        switch ($filter) {
        /* Check permissions for a single page, checking parents also. */
        case 'page':
            if (isset($permsCache[$in][$permission])) {
                return $permsCache[$in][$permission];
            }

            $page_name = $giapeto->getPagePath($in);
            $ids = substr($giapeto->getParentIdString($page_name) . ':' . $in, 1);
            $ids = array_reverse(explode(':', $ids));

            foreach ($ids as $page_id) {
                if ($this->hasPermission('giapeto:pages:' . $page_id, $userID, $permission)) {
                    /* Cache and $out set to true, but keep checking. */
                    $permsCache[$in][$permission] = true;
                    $out = true;
                }
            }
            break;

        /* Check permissions for subpages, assuming that the user has
         * passed checkPermissions to get to this point. */
        case 'subpages':
            $ids = join(';', array_keys($in));
            if (isset($permsCache[$ids][$permission])) {
                /* We found some already cached perms, return that. */
                return $permsCache[$ids][$permission];
            }
            foreach ($in as $page_id => $in_val) {
                if (!$this->exists('giapeto:pages:' . $page_id) || $this->hasPermission('giapeto:pages:' . $page_id, $userID, $permission)) {
                    $permsCache[$ids][$permission][$page_id] = $in_val;
                    $out[$page_id] = $in_val;
                }
            }
            break;

        /* Filter out the pages from the site tree for which the user
         * does not have permission for. */
        case 'sitetree':
            foreach ($in as $page_id => $page_name) {
                $parent_ids = explode(':', $giapeto->getParentIdString($page_name));
                //$parent_ids = explode(':', $parent_ids);
                /*if ((!$this->exists('giapeto:pages:' . $page_id) ||
                        $this->hasPermission('giapeto:pages:' . $page_id, $userID, $permission)) {*/
                    $out[$page_id] = $page_name;
                /*}*/
            }
            break;

        default:
            /* For now default everything to false. */
            $out = false;
        }

        return $out;
    }

    /**
     * @see Perms::getPermission()
     */
    function &getPermission()
    {
        $args = func_get_args();
        return call_user_func_array(array($this->_perms, __FUNCTION__), $args);
    }

    /**
     * @see Perms::removePermission()
     */
    function removePermission()
    {
        $args = func_get_args();
        return call_user_func_array(array($this->_perms, __FUNCTION__), $args);
    }

    /**
     * @see Perms::hasPermission()
     */
    function hasPermission()
    {
        $args = func_get_args();
        return call_user_func_array(array($this->_perms, __FUNCTION__), $args);
    }

    /**
     * @see Perms::exists()
     */
    function exists()
    {
        $args = func_get_args();
        return call_user_func_array(array($this->_perms, __FUNCTION__), $args);
    }

}
